/**
 * Smooth Scroll for WordPress. Exclusively on Envato Market: http://codecanyon.net/user/42theme/portfolio?ref=42Theme
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2019 42Theme ( https://42theme.com ). All rights reserved.
 * @license      Envato License https://codecanyon.net/licenses/standard?ref=42theme
 * @author       Alexander Khmelnitskiy ( ak@42theme.com )
 * @support      support@42theme.com
 **/

( function ( $ ) {

    "use strict";

    // noinspection JSDeprecatedSymbols
    /** Run as soon as the page DOM becomes safe to manipulate. */
    $( document ).ready(function() {

        let $presetName = $( '.t42-ss-preset-name' );

        /** Show messages if we have some. */
        ShowAdminMessages();

        /** Enable current preset button on page loading. */
        if ( $presetName.val() ) {
            $( '.t42-load-preset-box button' ).removeClass( 't42-active' );
            $( '.t42-load-preset-box ' + $presetName.val() ).addClass( 't42-active' );
        }

        /** Set Default preset. */
        $( '.t42-load-preset-box .t42-default-btn' ).on( 'click', {
            'stepSize': 100,
            'animationTime': 400,
            'accelerationScale': 3,
            'accelerationDelta': 50,
            'pulseScale': 4,
            'arrowStep': 50,
            'presetName': '.t42-default-btn',
            'presetButton': '.t42-default-btn'
        }, SetPresetValues );

        /** Set iPhone preset. */
        $( '.t42-load-preset-box .t42-iphone-btn' ).on( 'click', {
            'stepSize': 120,
            'animationTime': 600,
            'accelerationScale': 3,
            'accelerationDelta': 50,
            'pulseScale': 3,
            'arrowStep': 50,
            'presetName': '.t42-iphone-btn',
            'presetButton': '.t42-iphone-btn'
        }, SetPresetValues );

        /** Set Opera preset. */
        $( '.t42-load-preset-box .t42-opera-btn' ).on( 'click', {
            'stepSize': 120,
            'animationTime': 120,
            'accelerationScale': 3,
            'accelerationDelta': 50,
            'pulseScale': 3,
            'arrowStep': 50,
            'presetName': '.t42-opera-btn',
            'presetButton': '.t42-opera-btn'
        }, SetPresetValues );

        /** Set IE preset. */
        $( '.t42-load-preset-box .t42-ie-btn' ).on( 'click', {
            'stepSize': 120,
            'animationTime': 60,
            'accelerationScale': 3,
            'accelerationDelta': 50,
            'pulseScale': 3,
            'arrowStep': 50,
            'presetName': '.t42-ie-btn',
            'presetButton': '.t42-ie-btn'
        }, SetPresetValues );

        /** Set Custom preset. */
        $( '.t42-load-preset-box .t42-custom-btn' ).on( 'click', {
            'stepSize': 100,
            'animationTime': 400,
            'accelerationScale': 3,
            'accelerationDelta': 50,
            'pulseScale': 4,
            'arrowStep': 50,
            'presetName': '.t42-custom-btn',
            'presetButton': '.t42-custom-btn'
        }, SetPresetValues );

        /**
         * Set preset values in inputs.
         **/
        function SetPresetValues( event ) {

            /** Set preset values to inputs. */
            $( '.t42-ss-step-size' ).val( event.data.stepSize );
            $( '.t42-ss-animation-time' ).val( event.data.animationTime );
            $( '.t42-ss-acceleration-scale' ).val( event.data.accelerationScale );
            $( '.t42-ss-acceleration-delta' ).val( event.data.accelerationDelta );
            $( '.t42-ss-pulse-scale' ).val( event.data.pulseScale );
            $( '.t42-ss-arrow-step' ).val( event.data.arrowStep );
            $( '.t42-ss-keyboard-support' ).prop( 'checked', true );
            $( '.t42-ss-pulse-algorithm' ).prop( 'checked', true );
            $( '.t42-ss-all-browsers' ).prop( 'checked', true );

            /** Set preset name to hidden input. */
            $presetName.val( event.data.presetName );

            /** Deactivate all preset buttons. */
            $( '.t42-load-preset-box button' ).removeClass( 't42-active' );

            /** Activate preset button that was just pressed. */
            $( event.data.presetButton ).addClass( 't42-active' );
        }

        /**
         * Show popup with admin messages.
         **/
        function ShowAdminMessages() {

            let messages;

            /** If there is no data from backend. */
            // noinspection JSUnresolvedVariable
            if ( typeof window.t42SmoothScroll === 'undefined' ) {
                return;
            }

            /** If there is no messages. */
            // noinspection JSUnresolvedVariable
            if ( ! window.t42SmoothScroll.hasOwnProperty( 'messages' ) ) {
                return;
            }

            // noinspection JSUnresolvedVariable
            /** Get messages. */
            messages = window.t42SmoothScroll.messages;

            /** Show each message with their params. */
            messages.forEach( function( message ) {
                // noinspection JSUnresolvedVariable, JSHint
                iziToast.show( {
                    title: message.title,
                    message: message.message,
                    icon: message.icon,
                    color: message.color,
                    timeout: message.timeout,
                } );
            } ); // END messages.forEach
        } // END ShowAdminMessages

        /** On any settings change - activate Custom preset. */
        $( '.t42-tab-general input[type="number"], .t42-tab-general input[type="checkbox"]' ).on( 'change', function () {
            $presetName.val( '.t42-custom-btn' );
            $( '.t42-load-preset-box button' ).removeClass( 't42-active' );
            $( '.t42-load-preset-box .t42-custom-btn' ).addClass( 't42-active' );
        } );

        /** Open activation tab. */
        $( '.t42-activation-status .t42-activate-btn' ).on( 'click', function () {
            UIkit2t42.switcher( 'ul.t42-tab' ).show( 2 );
        } );


    } ); // END $( document ).ready();

    /**
     * Set current tab from sessionStorage value.
     **/
    function SetCurrentTab() {
        /** Get tab index from sessionStorage. */
        let t42SmoothScrollSelectedTab = sessionStorage.getItem( 't42SmoothScrollSelectedTab' );

        /** Or select first tab. */
        if ( ! t42SmoothScrollSelectedTab ) {
            t42SmoothScrollSelectedTab = 0;
        }

        /** Select previous tab. */
        UIkit2t42.switcher( 'ul.t42-tab' ).show( t42SmoothScrollSelectedTab ); // jshint ignore:line

        /** Scroll to  previously stored position. */
        setTimeout( SetScrollPosition, 500 );

    }

    /**
     * Set scroll position from sessionStorage value.
     **/
    function SetScrollPosition() {

        /** Get scroll position from sessionStorage. */
        let t42SmoothScrollScrollTop = sessionStorage.getItem( 't42SmoothScrollScrollTop' );

        /** Or set to top. */
        if ( ! t42SmoothScrollScrollTop ) {
            t42SmoothScrollScrollTop = 0;
        }

        /** Scroll to previous position. */
        $('.t42-settings-content').scrollTop( t42SmoothScrollScrollTop );

    }

    /**
     * Remember selected tab in sessionStorage.
     **/
    function RememberCurrentTab() {
        /** Get current selected tab. */
        let t42SmoothScrollSelectedTab = $( 'ul.t42-tab li.t42-active' ).index();

        /** Save value in sessionStorage. */
        window.sessionStorage.setItem( 't42SmoothScrollSelectedTab', t42SmoothScrollSelectedTab );
    }

    /**
     * Remember current scroll position in sessionStorage.
     **/
    function RememberScrollPosition() {
        /** Get current scroll position. */
        let t42SmoothScrollScrollTop = jQuery('.t42-settings-content').scrollTop();

        /** Save value in sessionStorage. */
        window.sessionStorage.setItem( 't42SmoothScrollScrollTop', t42SmoothScrollScrollTop );
    }

    /** Set previously stored tab. */
    $( window ).on('load', function () {

        /** Select previously stored tab. */
        SetCurrentTab();

        /** Remember selected tab on every tab change. */
        $( 'ul.t42-tab' ).on( 'show.uk.switcher', RememberCurrentTab );

        /** Remember scroll position on every scroll change. */
        $( '.t42-settings-content' ).on( 'scroll', RememberScrollPosition );

    } );

} ( jQuery ) );