<?php
/**
 * Smooth Scroll for WordPress. Exclusively on Envato Market: http://codecanyon.net/user/42theme/portfolio?ref=42Theme
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2019 42Theme ( https://42theme.com ). All rights reserved.
 * @license      Envato License https://codecanyon.net/licenses/standard?ref=42theme
 * @author       Alexander Khmelnitskiy ( ak@42theme.com )
 * @support      support@42theme.com
 **/

namespace T42\SmoothScroll;

use T42\SmoothScroll as SmoothScroll;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to render settings fields.
 *
 * @since 2.0.0
 * @author Alexander Khmelnitskiy ( ak@42theme.com )
 **/
final class Settings {

	/**
	 * SmoothScroll Plugin settings.
	 *
	 * @var array()
	 * @since 2.0.0
	 **/
	public $options = [];

	/**
	 * The one true Settings.
	 *
	 * @var Settings
	 * @since 2.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Settings instance.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Get plugin settings. */
		$this->options = $this->get_options();

	}

	/**
	 * Render head part for Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_head() {
		?>
        <div class="t42-header">
            <div>
                <h1>
                    <img class="t42-plugin-logo" src="<?php echo esc_attr( SmoothScroll::$url ) . 'images/sign-logo.svg'; ?>" alt="<?php esc_html_e( SmoothScroll::$name ); ?>">
					<?php esc_html_e( SmoothScroll::$name ); ?>
                    <sup><?php echo esc_html__( 'ver.', 't42-smoothscroll' ) . esc_html( SmoothScroll::$version ); ?></sup>
                </h1>

                <?php if ( ! PluginActivation::get_instance()->is_activated() ) : ?>
                    <div class="t42-activation-status">
                        <i class="t42-icon-warning"></i>
                        <?php esc_html_e( 'Your copy of Smooth Scroll plugin is not activated.', 't42-smoothscroll' ); ?>
                        <a href="#" class="t42-button t42-activate-btn"><?php esc_html_e( 'Activate Now', 't42-smoothscroll' ); ?></a>
                    </div>
                <?php endif; ?>

            </div>
        </div>
		<?php
	}

	/**
	 * Render tabs part for Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_tabs() {
	    ?>
        <div class="t42-tab-box" >
            <div>
                <ul class="t42-tab" data-t42-tab="{connect:'#t42-settings-tab', animation: 'fade', swiping: false}">
                    <li><a href=""><?php esc_html_e( 'General', 't42-smoothscroll' ); ?></a></li>
                    <li><a href=""><?php esc_html_e( 'Assignments', 't42-smoothscroll' ); ?></a></li>
                    <li><a href=""><?php esc_html_e( 'Activation', 't42-smoothscroll' ); ?></a></li>
                    <li><a href=""><?php esc_html_e( 'Status', 't42-smoothscroll' ); ?></a></li>
                    <li><a href=""><?php esc_html_e( 'Uninstall', 't42-smoothscroll' ); ?></a></li>
                </ul>

                <ul id="t42-settings-tab" class="t42-switcher t42-margin">
                    <li class="t42-tab-general">
						<?php settings_fields( 'SmoothScrollOptionsGroup' ); ?>
						<?php do_settings_sections( 'SmoothScrollOptionsGroup' ); ?>

                        <div class="t42-panel t42-panel-box">
                            <h2><?php esc_html_e( 'General Settings', 't42-smoothscroll' ); ?></h2>
                            <?php $SF = SettingsFields::get_instance(); ?>
							<?php $SF->load_preset(); ?>
							<?php $SF->step_size(); ?>
							<?php $SF->animation_time(); ?>
							<?php $SF->acceleration_scale(); ?>
							<?php $SF->acceleration_delta(); ?>
							<?php $SF->pulse_scale(); ?>
							<?php $SF->arrow_step(); ?>
							<?php $SF->keyboard_support(); ?>
							<?php $SF->pulse_algorithm(); ?>
							<?php $SF->all_browsers(); ?>
	                        <?php $SF->enable_admin(); ?>
                        </div>
                    </li>
                    <li class="t42-tab-assignments">
                        <div class="t42-panel t42-panel-box">
                            <h2><?php esc_html_e( 'Assignments Settings', 't42-smoothscroll' ); ?></h2>
                            <?php TabAssignments::get_instance()->render_assignments(); ?>
                        </div>
                    </li>
                    <li class="t42-tab-activation">
                        <div class="t42-panel t42-panel-box">
                            <h2><?php esc_html_e( 'Plugin Activation', 't42-smoothscroll' ); ?></h2>
                            <?php
                                /** Plugin Activation Tab. */
                                PluginActivation::get_instance()->render_pid();

                                /** Render FAQ accordion. */
                                PluginActivation::get_instance()->render_FAQ();
                            ?>
                        </div>
                    </li>
                    <li class="t42-tab-status">
                        <div class="t42-panel t42-panel-box">
                            <h2><?php esc_html_e( 'Status Reports', 't42-smoothscroll' ); ?></h2>
                            <?php
                            /** Get TabStatus instance. */
                            $status = TabStatus::get_instance();

                            /** Render "System Requirements". */
                            $status->render_system_requirements();

                            /** Render Privacy Notice. */
                            $status->render_privacy_notice();

                            ?>
                        </div>
                    </li>
                    <li class="t42-tab-uninstall">
                        <div class="t42-panel t42-panel-box">
                            <h2><?php esc_html_e( 'Uninstall Settings', 't42-smoothscroll' ); ?></h2>
                            <?php SettingsFields::get_instance()->field_delete_plugin(); ?>
                        </div>
                    </li>
                </ul>
            </div>
        </div>
        <?php
    }

	/**
	 * Render footer part for Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function render_foot() {
	    ?>
        <footer class="t42-bottom-fixed">
            <div class="t42-width-1-1">

                <div class="t42-flex t42-flex-middle t42-flex-space-between">
                    <div>
                        <button type="submit" name="submit" id="submit" class="t42-button t42-button-primary"><?php esc_html_e( 'Save Changes', 't42-smoothscroll' ); ?></button>
                    </div>
                    <div>
                        <ul class="t42-subnav t42-subnav-line">
                            <li><a href="<?php echo esc_url( EnvatoItem::$documentation_url ); ?>" target="_blank"><?php esc_html_e( 'Documentation', 't42-smoothscroll' ); ?></a></li>
                            <li><a href="<?php echo esc_url( EnvatoItem::$changelog_url ); ?>" target="_blank"><?php esc_html_e( 'Changelog', 't42-smoothscroll' ); ?></a></li>
                            <li><a href="https://codecanyon.net/user/42theme/portfolio?ref=42Theme" target="_blank"><?php esc_html_e( 'More Plugins', 't42-smoothscroll' ); ?></a></li>
                        </ul>
                    </div>
                </div>

            </div>
        </footer>
        <?php
	}

	/**
	 * Plugin Settings Page.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function render_options_page() {

		/** Security check. */
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		$activated = 'not-activated';
		if ( PluginActivation::get_instance()->is_activated() ) {
			$activated = 'activated';
        }

		?>
        <!--suppress HtmlUnknownTarget -->
        <form class="t42-form t42-form-horizontal" action='options.php' method='post'>
            <div class="t42-settings-container <?php esc_attr_e( $activated ); ?>">

                <div class="t42-settings-content">

                    <?php $this->render_head(); ?>

                    <?php $this->render_tabs(); ?>

                </div>

                <?php $this->render_foot(); ?>

            </div>
		</form>
		<?php
	}

	/**
	 * Get saved plugin settings or default values.
	 *
	 * @since 2.0.0
	 * @return array
	 **/
	public function get_options() {

		/** Options. */
		$options = get_option( 't42_smoothscroll_settings' );

		/** Default values. */
		$defaults = [
			'preset_name'                               => '.t42-default-btn', // Default preset.
			'step_size'                                 => 100, // Step size (px).
			'animation_time'                            => 400, // Animation time (ms).
			'acceleration_scale'                        => 3, // Acceleration scale.
			'acceleration_delta'                        => 50, // Acceleration delta (ms).
			'pulse_scale'                               => 4, // Pulse Scale.
			'arrow_step'                                => 50, // Arrow key step size (px).
			'assignments'                               => "{|matchingMethod|:1,|WPContent|:0,|WPContentVal|:||,|homePage|:0,|menuItems|:0,|menuItemsVal|:||,|dateTime|:0,|dateTimeStart|:||,|dateTimeEnd|:||,|userRoles|:0,|userRolesVal|:||,|URL|:0,|URLVal|:||,|devices|:0,|devicesVal|:||,|PHP|:0,|PHPVal|:||}", // Assignments
            'delete_plugin'                             => 'plugin', // Uninstall mode, only plugin.
			'envato_purchase_code_' . EnvatoItem::get_instance()->get_id()    => '',
		];

		/** Default values for checkboxes. */
		if ( $options === false ) {
			$defaults['keyboard_support'] = 'true'; // Enable keyboard support.
			$defaults['pulse_algorithm']  = 'true'; // Enable pulse algorithm.
			$defaults['all_browsers']     = 'true'; // Work only in WebKit browsers.
			$defaults['enable_admin']     = 'false'; // Work in admin area.
		} else {
			/** Set all to FALSE, if options have one, then wp_parse_args override false value. */
			$defaults['keyboard_support'] = 'false'; // Enable keyboard support.
			$defaults['pulse_algorithm']  = 'false'; // Enable pulse algorithm.
			$defaults['all_browsers']     = 'false'; // Work only in WebKit browsers.
			$defaults['enable_admin']     = 'false'; // Work in admin area.
		}

		/**
		 * The Purchase Code entered by the user must be stored in the wp_options table as a string
		 * using the following syntax:
		 *    option_name: envato_purchase_code_<item_id>
         *    option_value: <purchase_code>
         **/
        if ( isset( $options['envato_purchase_code_' . EnvatoItem::get_instance()->get_id()] ) ) {
            update_option( 'envato_purchase_code_' . EnvatoItem::get_instance()->get_id(), $options['envato_purchase_code_' . EnvatoItem::get_instance()->get_id()] );
        }

		return wp_parse_args( $options, $defaults );
	}

	/**
	 * Generate Settings Page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function settings_init() {

		$group_name = 'SmoothScrollOptionsGroup';
		$section_id = 't42_smoothscroll_settings_page_general_section';
		$option_name = 't42_smoothscroll_settings';

		/** Create settings section. */
		register_setting( $group_name, $option_name );
		add_settings_section( $section_id, '', null, $group_name );

	}

	/**
	 * Add plugin settings page.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function add_settings_page() {
		add_action( 'admin_menu', [$this, 'add_admin_menu'] );
		add_action( 'admin_init', [$this, 'settings_init'] );
	}

	/**
	 * Add admin menu for plugin settings.
	 *
	 * @since 1.1.0
	 * @access public
	 * @return void
	 **/
	public function add_admin_menu() {

		add_menu_page(
			esc_html__( 'Smooth Scroll Settings', 't42-smoothscroll' ),
			esc_html__( 'Smooth Scroll', 't42-smoothscroll' ),
			'manage_options',
			't42_smoothscroll_settings',
			[ $this, 'render_options_page' ],
			'data:image/svg+xml;base64,' . base64_encode( file_get_contents( SmoothScroll::$path . 'images/logo-menu.svg' ) ),
			'58.2852'// Always change digits after "." for different plugins.
		);
	}

	/**
	 * Main Settings Instance.
	 *
	 * Insures that only one instance of Settings exists in memory at any one time.
	 *
	 * @static
	 * @return Settings
	 * @since 2.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Settings ) ) {
			self::$instance = new Settings();
		}

		return self::$instance;

	}

} // End Class Settings.

