<?php
/**
 * Smooth Scroll for WordPress. Exclusively on Envato Market: http://codecanyon.net/user/42theme/portfolio?ref=42Theme
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2019 42Theme ( https://42theme.com ). All rights reserved.
 * @license      Envato License https://codecanyon.net/licenses/standard?ref=42theme
 * @author       Alexander Khmelnitskiy ( ak@42theme.com )
 * @support      support@42theme.com
 **/

namespace T42\SmoothScroll;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to render settings fields.
 *
 * @since 2.0.0
 * @author Alexander Khmelnitskiy ( ak@42theme.com )
 **/
final class SettingsFields {

	/**
	 * The one true SettingsFields.
	 *
	 * @var SettingsFields
	 * @since 2.0.0
	 **/
	private static $instance;

	/**
	 * Render "Delete Plugin" field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function field_delete_plugin() {

		/** Get plugins settings. */
		$delete_plugin = Settings::get_instance()->options['delete_plugin'];

		$options = [
			'plugin' => esc_html__( 'Delete plugin only', 't42-smoothscroll' ),
			'plugin+settings' => esc_html__( 'Delete plugin and settings', 't42-smoothscroll' ),
		];

		/** Prepare description. */
		$helper_text = esc_html__( 'Choose which data to delete upon using the "Delete" action in the "Plugins" admin page.', 't42-smoothscroll' );

		/** Render select. */
		SettingsUI::get_instance()->render_select(
			$options,
			$delete_plugin, // Selected option.
			esc_html__('Delete plugin', 't42-smoothscroll' ),
			$helper_text,
			['name' => 't42_smoothscroll_settings[delete_plugin]']
		);

	}

	/**
	 * Render Step Size field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function step_size() {

		$name = 't42_smoothscroll_settings[step_size]';
		$value = Settings::get_instance()->options['step_size'];
		$label = esc_html__( 'Step size (px)', 't42-smoothscroll' );
		$desc = esc_html__( 'How much? Higher means more scrolling for each mouse event.', 't42-smoothscroll' );
		$class = 't42-ss-step-size t42-form-large t42-form-width-small';

		SettingsUI::get_instance()->render_number( $name, $value, $class, $label, $desc, 0, 999 );

	}

	/**
	 * Render Animation time field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function animation_time() {

		$name = 't42_smoothscroll_settings[animation_time]';
		$value = Settings::get_instance()->options['animation_time'];
		$label = esc_html__( 'Animation time (ms)', 't42-smoothscroll' );
		$desc = esc_html__( 'How quick? Higher number stretches animation.', 't42-smoothscroll' );
		$class = 't42-ss-animation-time t42-form-large t42-form-width-small';

		SettingsUI::get_instance()->render_number( $name, $value, $class, $label, $desc, 0, 999 );

	}

	/**
	 * Render Acceleration Scale field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function acceleration_scale() {

		$name = 't42_smoothscroll_settings[acceleration_scale]';
		$value = Settings::get_instance()->options['acceleration_scale'];
		$label = esc_html__( 'Acceleration scale', 't42-smoothscroll' );
		$desc = esc_html__( 'How swift? How much scrolling can accelerate.', 't42-smoothscroll' );
		$class = 't42-ss-acceleration-scale t42-form-large t42-form-width-small';

		SettingsUI::get_instance()->render_number( $name, $value, $class, $label, $desc, 0, 999 );

	}

	/**
	 * Render Acceleration Delta field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function acceleration_delta() {

		$name = 't42_smoothscroll_settings[acceleration_delta]';
		$value = Settings::get_instance()->options['acceleration_delta'];
		$label = esc_html__( 'Acceleration scale', 't42-smoothscroll' );
		$desc = esc_html__( 'Time between scroll events. Lower means acceleration kicks in more frequently.', 't42-smoothscroll' );
		$class = 't42-ss-acceleration-delta t42-form-large t42-form-width-small';

		SettingsUI::get_instance()->render_number( $name, $value, $class, $label, $desc, 0, 99 );

	}

	/**
	 * Render Pulse Scale field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function pulse_scale() {

		$name = 't42_smoothscroll_settings[pulse_scale]';
		$value = Settings::get_instance()->options['pulse_scale'];
		$label = esc_html__( 'Acceleration scale', 't42-smoothscroll' );
		$desc = esc_html__( 'Ratio of "tail" to "acceleration". It affects how long the acceleration part is.', 't42-smoothscroll' );
		$class = 't42-ss-pulse-scale t42-form-large t42-form-width-small';

		SettingsUI::get_instance()->render_number( $name, $value, $class, $label, $desc, 0, 99 );

	}

	/**
	 * Render Arrow key step size field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function arrow_step() {

		$name = 't42_smoothscroll_settings[arrow_step]';
		$value = Settings::get_instance()->options['arrow_step'];
		$label = esc_html__( 'Acceleration scale', 't42-smoothscroll' );
		$desc = esc_html__( 'Higher means more scrolling for each arrow key press.', 't42-smoothscroll' );
		$class = 't42-ss-arrow-step t42-form-large t42-form-width-small';

		SettingsUI::get_instance()->render_number( $name, $value, $class, $label, $desc, 0, 999 );

	}

	/**
	 * Render Keyboard Support field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function keyboard_support() {

		$name = 't42_smoothscroll_settings[keyboard_support]';
		$label = esc_html__( 'Enable keyboard support', 't42-smoothscroll' );
		$class = 't42-ss-keyboard-support';
		$checked = checked( filter_var( Settings::get_instance()->options['keyboard_support'], FILTER_VALIDATE_BOOLEAN ), true, false );
		$desc = esc_html__( 'Adds SmoothScroll for Arrow keys, PgUp/PgDown, Spacebar, Home/End.', 't42-smoothscroll' );

		SettingsUI::get_instance()->render_checkbox( $name, $label, $checked, $class, $desc );

	}

	/**
	 * Render Pulse algorithm field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function pulse_algorithm() {

		$name = 't42_smoothscroll_settings[pulse_algorithm]';
		$label = esc_html__( 'Enable pulse algorithm', 't42-smoothscroll' );
		$class = 't42-ss-pulse-algorithm';
		$checked = checked( filter_var( Settings::get_instance()->options['pulse_algorithm'], FILTER_VALIDATE_BOOLEAN ), true, false );
		$desc = esc_html__( 'Adds easing to the animations.', 't42-smoothscroll' );

		SettingsUI::get_instance()->render_checkbox( $name, $label, $checked, $class, $desc );

	}

	/**
	 * Render Enable for All Browsers field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function all_browsers() {

		$name = 't42_smoothscroll_settings[all_browsers]';
		$label = esc_html__( 'Enable for all browsers', 't42-smoothscroll' );
		$class = 't42-ss-all-browsers';
		$checked = checked( filter_var( Settings::get_instance()->options['all_browsers'], FILTER_VALIDATE_BOOLEAN ), true, false );
		$desc = esc_html__( 'By default SmoothScroll works only in WebKit based browsers. This option enable SmoothScroll in all browsers, such as Firefox.', 't42-smoothscroll' );

		SettingsUI::get_instance()->render_checkbox( $name, $label, $checked, $class, $desc );

	}

	/**
	 * Render Enable for Admin field.
	 *
	 * @since 1.1.0
	 * @access public
	 **/
	public function enable_admin() {

		$name = 't42_smoothscroll_settings[enable_admin]';
		$label = esc_html__( 'Enable for Admin', 't42-smoothscroll' );
		$class = 't42-ss-enable-admin';
		$checked = checked( filter_var( Settings::get_instance()->options['enable_admin'], FILTER_VALIDATE_BOOLEAN ), true, false );
		$desc = esc_html__( 'Enable SmoothScroll for works in WordPress admin area too.', 't42-smoothscroll' );

		SettingsUI::get_instance()->render_checkbox( $name, $label, $checked, $class, $desc );

	}

	/**
	 * Render Load preset: Default, iPhone, Opera, IE. field.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function load_preset() { ?>

		<div class="t42-load-preset-header">
			<div class="t42-title"><?php esc_html_e( 'Load preset', 't42-smoothscroll' ); ?></div>
			<div class="t42-button-box">
				<ul class="t42-load-preset-box">
					<li>
						<button type="button" class="t42-default-btn t42-button t42-button-secondary t42-active">
							<?php esc_html_e( 'Default', 't42-smoothscroll' ); ?>
						</button>
					</li>
					<li>
						<button type="button" class="t42-iphone-btn t42-button t42-button-secondary">
							<?php esc_html_e( 'iPhone', 't42-smoothscroll' ); ?>
						</button>
					</li>
					<li>
						<button type="button" class="t42-opera-btn t42-button t42-button-secondary">
							<?php esc_html_e( 'Opera', 't42-smoothscroll' ); ?>
						</button>
					</li>
					<li>
						<button type="button" class="t42-ie-btn t42-button t42-button-secondary">
							<?php esc_html_e( 'IE', 't42-smoothscroll' ); ?>
						</button>
					</li>
					<li>
						<button type="button" class="t42-custom-btn t42-button t42-button-secondary">
							<?php esc_html_e( 'Custom', 't42-smoothscroll' ); ?>
						</button>
					</li>
				</ul>
			</div>
		</div>

		<input class="t42-ss-preset-name"
		       type='hidden'
		       name='t42_smoothscroll_settings[preset_name]'
		       value='<?php echo esc_attr( Settings::get_instance()->options['preset_name'] ); ?>'>
		<?php
	}

	/**
	 * Main SettingsFields Instance.
	 *
	 * Insures that only one instance of SettingsFields exists in memory at any one time.
	 *
	 * @static
	 * @return SettingsFields
	 * @since 2.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof SettingsFields ) ) {
			self::$instance = new SettingsFields();
		}

		return self::$instance;

	}

} // End Class SettingsFields.

