<?php
/**
 * Smooth Scroll for WordPress. Exclusively on Envato Market: http://codecanyon.net/user/42theme/portfolio?ref=42Theme
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2019 42Theme ( https://42theme.com ). All rights reserved.
 * @license      Envato License https://codecanyon.net/licenses/standard?ref=42theme
 * @author       Alexander Khmelnitskiy ( ak@42theme.com )
 * @support      support@42theme.com
 **/

namespace T42\SmoothScroll;

use T42\SmoothScroll as SmoothScroll;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to create shortcodes.
 *
 * @since 2.0.0
 * @author Alexander Khmelnitskiy ( ak@42theme.com )
 **/
final class Shortcodes {

	/**
	 * The one true Shortcodes.
	 *
	 * @var Shortcodes
	 * @since 2.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Shortcodes instance.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Add shortcode [disable_smoothscroll] */
		add_shortcode( 't42_disable_smooth_scroll', [$this, 'disable_smoothscroll_shortcode'] );

	}

	/**
	 * Shortcode to disable plugin for certain pages.
	 * Use [t42_disable_smooth_scroll].
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	function disable_smoothscroll_shortcode() {

		/** Checks if plugin should work on this page. */
		if ( ! TabAssignments::get_instance()->display() ) { return; }

		wp_enqueue_script( 't42-smoothscroll-destroyer-js', SmoothScroll::$url . 'js/smoothscroll-destroyer' . SmoothScroll::$suffix . '.js', array(), SmoothScroll::$version, true );

		return;
	}

	/**
	 * Main Shortcodes Instance.
	 *
	 * Insures that only one instance of Shortcodes exists in memory at any one time.
	 *
	 * @static
	 * @return Shortcodes
	 * @since 2.0.0
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Shortcodes ) ) {
			self::$instance = new Shortcodes();
		}

		return self::$instance;

	}

} // End Class Shortcodes.

