<?php
/**
 * Plugin Name: Smooth Scroll
 * Plugin URI: https://codecanyon.net/item/smooth-scroll-scroll-site-without-jerky-and-clunky-effects-wordpress-version/19782948?ref=42Theme
 * Description: Smooth Scroll plugin will make your WordPress site cool by making the scroll movement of the mouse wheel smooth. Making Google Chrome And WebKit Based Browsers Scroll Smoother.
 * Version: 2.0.1
 * Author: پرشین اسکریپت
 * Author URI: https://persianscript.ir
 **/

/**
 * Smooth Scroll for WordPress. Exclusively on Envato Market: http://codecanyon.net/user/42theme/portfolio?ref=42Theme
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2019 42Theme ( https://42theme.com ). All rights reserved.
 * @license      Envato License https://codecanyon.net/licenses/standard?ref=42theme
 * @author       Alexander Khmelnitskiy ( ak@42theme.com )
 * @support      support@42theme.com
 **/

namespace T42;

use T42\SmoothScroll\Helper;
use T42\SmoothScroll\Settings;
use T42\SmoothScroll\EnvatoItem;
use T42\SmoothScroll\Shortcodes;
use T42\SmoothScroll\PluginHelper;
use T42\SmoothScroll\MessageHelper;
use T42\SmoothScroll\PluginUpdater;
use T42\SmoothScroll\TabAssignments;
use T42\SmoothScroll\PluginActivation;
use T42\SmoothScroll\CheckCompatibility;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';

/**
 * SINGLETON: Core class used to implement a SmoothScroll plugin.
 *
 * This is used to define core logic of plugin.
 *
 * @since 2.0.0
 **/
final class SmoothScroll {

	/**
     * Plugin version.
	 *
	 * @string $version
	 * @since 2.0.0
	 **/
	public static $version = '';

	/**
	 * Plugin name.
	 *
	 * @string $version
	 * @since 2.0.0
	 **/
	public static $name = '';

	/**
	 * Use minified libraries if SCRIPT_DEBUG is turned off.
	 *
	 * @since 2.0.0
	 **/
	public static $suffix = '';

	/**
	 * URL (with trailing slash) to plugin folder.
	 *
	 * @var string
	 * @since 2.0.0
	 **/
	public static $url = '';

	/**
	 * PATH to plugin folder.
	 *
	 * @var string
	 * @since 2.0.0
	 **/
	public static $path = '';

	/**
	 * Plugin base name.
	 *
	 * @var string
	 * @since 1.0.0
	 **/
	public static $basename = '';

	/**
	 * Plugin admin menu base.
	 *
	 * @var string
	 * @since 2.0.1
	 **/
	public static $menu_base;

	/**
	 * The one true SmoothScroll.
	 *
	 * @var SmoothScroll
	 * @since 1.1.0
	 **/
	private static $instance;

	/**
	 * Sets up a new plugin instance.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Initialize main variables. */
	    $this->initialization();

	}

	/**
	 * Setup the plugin.
	 *
	 * @since 2.0.1
	 * @access public
	 *
	 * @return void
	 **/
	public function setup() {

		/** Do critical initial checks. */
		if ( ! CheckCompatibility::get_instance()->do_initial_checks( true ) ) { return; }

		/** Send install Action to our host. */
		self::send_install_action();

		/** Define hooks that runs on both the front-end as well as the dashboard. */
		$this->both_hooks();

		/** Define public hooks. */
		$this->public_hooks();

		/** Define admin hooks. */
		$this->admin_hooks();

	}

	/**
	 * Define hooks that runs on both the front-end as well as the dashboard.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
	private function both_hooks() {

		/** Load translation. */
		add_action( 'plugins_loaded', [$this, 'load_textdomain'] );

	}

	/**
	 * Return plugin version.
	 *
	 * @return string
	 * @since 2.0.0
	 * @access public
	 **/
	public function get_version() {

		return self::$version;

	}

	/**
	 * Initialize main variables.
	 *
	 * @since 2.0.0
	 * @access public
     * @return void
	 **/
	public function initialization() {

		/** Get Plugin version. */
		$plugin_data = $this->get_plugin_data();
		self::$version = $plugin_data['Version'];

		/** Plugin name. */
		self::$name = $plugin_data['Name'];

		/** Gets the plugin URL (with trailing slash). */
		self::$url = plugin_dir_url( __FILE__ );

		/** Gets the plugin PATH. */
		self::$path = plugin_dir_path( __FILE__ );

		/** Use minified libraries if SCRIPT_DEBUG is turned off. */
		self::$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';

		/** Set plugin basename. */
		self::$basename = plugin_basename( __FILE__ );

		/** Plugin settings page base. */
		self::$menu_base = 'toplevel_page_t42_smoothscroll_settings';

	}

	/**
	 * Register all of the hooks related to the admin area functionality.
	 *
	 * @since 2.0.0
	 * @access private
     * @return void
	 **/
    private function admin_hooks() {

	    /** Work only in admin area. */
	    if ( ! is_admin() ) { return; }

	    /** Run PluginHelper. */
	    PluginHelper::get_instance();

	    /** Add plugin setting page. */
        Settings::get_instance()->add_settings_page();

	    /** Add admin styles. */
	    add_action( 'admin_enqueue_scripts', [$this, 'admin_styles' ] );

	    /** Add admin javascript. */
	    add_action( 'admin_enqueue_scripts', [$this, 'admin_scripts' ] );

	    /** Remove "Thank you for creating with WordPress" and WP version only from plugin settings page. */
	    add_action( 'admin_enqueue_scripts', [$this, 'remove_wp_copyrights'] );

	    /** Remove all "third-party" notices from plugin settings page. */
	    add_action( 'in_admin_header', [$this, 'remove_all_notices'], 1000);

	    /** Plugin Assignments tab. */
	    TabAssignments::get_instance();

	    /** Plugin update mechanism enable only if plugin have Envato ID. */
	    $plugin_id = EnvatoItem::get_instance()->get_id();
	    if ( (int)$plugin_id > 0 ) {
		    PluginUpdater::get_instance();
	    }

	    /** Add "Settings Saved" message. */
	    $this->add_message_settings_saved();

	    /** Add "Activation" message. */
	    add_action( 'update_option_t42_smoothscroll_settings', [$this, 'add_message_activation'], 10, 2 );

    }

	/**
	 * Return current plugin metadata.
	 *
	 * @since 2.0.1
	 * @access public
	 *
	 * @return array {
	 *     Plugin data. Values will be empty if not supplied by the plugin.
	 *
	 *     @type string $Name        Name of the plugin. Should be unique.
	 *     @type string $Title       Title of the plugin and link to the plugin's site (if set).
	 *     @type string $Description Plugin description.
	 *     @type string $Author      Author's name.
	 *     @type string $AuthorURI   Author's website address (if set).
	 *     @type string $Version     Plugin version.
	 *     @type string $TextDomain  Plugin textdomain.
	 *     @type string $DomainPath  Plugins relative directory path to .mo files.
	 *     @type bool   $Network     Whether the plugin can only be activated network-wide.
	 *     @type string $RequiresWP  Minimum required version of WordPress.
	 *     @type string $RequiresPHP Minimum required version of PHP.
	 * }
	 **/
	public function get_plugin_data() {

		if ( ! function_exists('get_plugin_data') ) {

			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );

		}

		return get_plugin_data( __FILE__ );

	}

	/**
	 * Fires after the value of envato_purchase_code_{item_id} option has been successfully updated.
	 *
	 * @param $old_value mixed - The old option value.
	 * @param $value mixed - The new option value.
	 *
	 * @since  2.0.0
	 * @access private
	 * @return void
	 **/
    public function add_message_activation( $old_value, $value ) {

    	/** Work only if purchase code was changed. */
    	if (
    		$old_value['envato_purchase_code_' . EnvatoItem::get_instance()->get_id()] ===
		    $value['envato_purchase_code_' . EnvatoItem::get_instance()->get_id()] ) {
    		return;
	    }

	    PluginActivation::get_instance()->reset_temporary_activation();

	    if ( PluginActivation::get_instance()->is_activated( $value['envato_purchase_code_' . EnvatoItem::get_instance()->get_id()] ) ) {

		    /** Add "Activation success" message. */
		    MessageHelper::get_instance()->add_message([
			    'title' => esc_html__( 'Activation Success.', 't42-smoothscroll' ),
			    'message' => esc_html__( 'Thank you for activating.', 't42-smoothscroll' ),
			    'icon' => 'ico-check',
			    'color' => 'green',
			    'timeout' => 7500
		    ]);

	    } else {

		    /** Add "Activation failed" message. */
		    MessageHelper::get_instance()->add_message([
			    'title' => esc_html__( 'Activation failed.', 't42-smoothscroll' ),
			    'message' => esc_html__( 'Invalid purchase code.', 't42-smoothscroll' ),
			    'icon' => 'ico-ban',
			    'color' => 'red',
			    'timeout' => 7500
		    ]);

	    }

    }

	/**
	 * Add "Settings Saved" message.
	 *
	 * @since 2.0.0
	 * @access private
	 * @return void
	 **/
    public function add_message_settings_saved() {

    	/** Work only if settings was updated. */
	    if ( ! isset( $_GET['settings-updated'] ) ) { return; }

	    $settings_updated = filter_input( INPUT_GET, 'settings-updated' );
	    $settings_updated = filter_var( $settings_updated, FILTER_VALIDATE_BOOLEAN );

	    /** Some error occurred on settings save. */
	    if ( ! $settings_updated ) { return; }

	    /** Add "Settings saved" message. */
	    MessageHelper::get_instance()->add_message([
		    'title' => esc_html__( 'Settings saved.', 't42-smoothscroll' ),
		    'message' => esc_html__( 'Plugin settings saved successfully.', 't42-smoothscroll' ),
		    'icon' => 'ico-info',
		    'color' => 'blue',
		    'timeout' => 7000
	    ]);

    }

	/**
	 * Register all of the hooks related to the public-facing functionality.
	 *
	 * @since 2.0.0
	 * @access private
     * @return void
	 **/
	private function public_hooks() {

		/** Work only on frontend area. */
		if ( is_admin() ) { return; }

		/** Add plugin script. */
		add_action( 'wp_enqueue_scripts', [$this, 'enqueue_scripts'] );

		/** Add shortcodes. */
		Shortcodes::get_instance();

    }

	/**
	 * Remove all other notices.
	 *
	 * @since 1.0.0
	 * @access public
	 **/
	public function remove_all_notices() {

		/** Work only on plugin settings page. */
		$screen = get_current_screen();
		if ( null === $screen || $screen->base !== self::$menu_base ) { return; }

		/** Remove other notices. */
		remove_all_actions( 'admin_notices' );
		remove_all_actions( 'all_admin_notices' );

	}

	/**
	 * Remove "Thank you for creating with WordPress" and WP version only from plugin settings page.
	 *
	 * @since 1.0.0
	 * @access private
	 * @return void
	 **/
	public function remove_wp_copyrights() {

		/** Remove "Thank you for creating with WordPress" and WP version from plugin settings page. */
		$screen = get_current_screen(); // Get current screen.

		/** Smooth Scroll Settings Page. */
		if ( null !== $screen && $screen->base === self::$menu_base ) {
			add_filter( 'admin_footer_text', '__return_empty_string', 11 );
			add_filter( 'update_footer', '__return_empty_string', 11 );
		}

	}

	/**
	 * Load the plugin text domain for translation.
	 *
	 * @since 1.0.0
	 * @access public
     * @return void
	 **/
	public function load_textdomain() {
	    
		load_plugin_textdomain( 't42-smoothscroll', false, self::$path . 'languages/' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @return void
	 * @since   1.0.0
	 **/
	public function enqueue_scripts() {

		/** Checks if plugin should work on this page. */
		if ( ! TabAssignments::get_instance()->display() ) { return; }

		/** Enqueue SmoothScroll for frontend. */
		$this->enqueue_smoothscroll();

	}

	/**
	 * Enqueue SmoothScroll scripts for frontend or backend.
	 *
	 * @return void
	 * @since   1.0.0
	 **/
	public function enqueue_smoothscroll() {

		/** Get plugin settings. */
		$options = Settings::get_instance()->options;

		/** Use minified libraries if SCRIPT_DEBUG is turned off. */
		wp_enqueue_script( 't42-smoothscroll', self::$url . 'js/smoothscroll' . self::$suffix . '.js', [], self::$version, true );

		/** Custom scroll settings. */
		$keyboardSupport = $options['keyboard_support'];
		$pulseAlgorithm  = $options['pulse_algorithm'];
		$allBrowsers     = $options['all_browsers'];

		$js = "
			let t42SmoothScrollTimer = setInterval( function() {
				if ( typeof SmoothScroll === 'undefined' ) { return; }
				
				SmoothScroll( {
					stepSize: {$options['step_size']},
					animationTime: {$options['animation_time']},
					accelerationMax: {$options['acceleration_scale']},
					accelerationDelta: {$options['acceleration_delta']},
					pulseScale: {$options['pulse_scale']},
					arrowScroll: {$options['arrow_step']},
					keyboardSupport: {$keyboardSupport},
					pulseAlgorithm: {$pulseAlgorithm}
				 } );
				SmoothScroll.restart();
				
				clearInterval( t42SmoothScrollTimer );
				
			}, 200 );
		         
		";

		/** After SmoothScroll script. */
		wp_add_inline_script( 't42-smoothscroll', $js );

		/** Before SmoothScroll script. */
		$js = "window.t42SmoothScrollAllBrowsers = {$allBrowsers};" . PHP_EOL;
		wp_add_inline_script( 't42-smoothscroll', $js, 'before' );

	}

	/**
	 * Add CSS for admin area.
	 *
	 * @param string $hook - The current admin page.
	 *
	 * @return void
	 * @since   1.1.0
	 **/
	public function admin_styles( $hook ) {

		/** Load styles for plugin settings page. */
		if ( self::$menu_base === $hook ) {

			wp_enqueue_style( 't42-iziToast', self::$url . 'css/iziToast' . self::$suffix . '.css', [], self::$version );
			wp_enqueue_style( 't42-smoothscroll-uikit', self::$url . 'css/uikit' . self::$suffix . '.css', [], self::$version );
			wp_enqueue_style( 't42-smoothscroll-admin', self::$url . 'css/admin' . self::$suffix . '.css', [], self::$version );

		} elseif ( 'plugin-install.php' === $hook ) {

			/** Styles only for our plugin. */
			if ( isset( $_GET['plugin'] ) AND $_GET['plugin'] === 't42-smoothscroll' ) {
				wp_enqueue_style( 't42-smoothscroll-plugin-install', self::$url . 'css/plugin-install' . self::$suffix . '.css', [], self::$version );
			}

		}

	}

	/**
	 * Add JS for admin area.
	 *
	 * @param string $hook - The current admin page.
	 *
	 * @return void
	 * @since   1.1.0
	 **/
	public function admin_scripts( $hook ) {

		/** Scripts only for plugin settings page. */
		if ( $hook === self::$menu_base ) {
			wp_enqueue_script( 't42-iziToast', self::$url . 'js/iziToast' . self::$suffix . '.js', [], self::$version, true );
			wp_enqueue_script( 't42-smoothscroll-uikit', self::$url . 'js/uikit' . self::$suffix . '.js', ['jquery'], self::$version, true );
			wp_enqueue_script( 't42-smoothscroll-uikit-accordion', self::$url . 'js/accordion.uikit' . self::$suffix . '.js', ['jquery', 't42-smoothscroll-uikit'], self::$version, true );
			wp_enqueue_script( 't42-smoothscroll-admin', self::$url . 'js/admin' . self::$suffix . '.js', ['jquery'], self::$version, true );

			/** Pass admin messages to JavaScript. */
			$this->pass_messages_to_js();
		}

		/** If SmoothScroll enabled for admin. */
		if ( filter_var( Settings::get_instance()->options['enable_admin'], FILTER_VALIDATE_BOOLEAN ) ) {

			/** Enqueue SmoothScroll for frontend. */
			$this->enqueue_smoothscroll();

		}

	}

	/**
	 * Pass admin messages to JavaScript.
	 *
	 * @static
	 * @since 2.0.0
	 **/
	private function pass_messages_to_js() {

		/** Pass admin messages to JavaScript. */
		$messages = MessageHelper::get_instance()->get_messages();
		if ( count( $messages ) ) {
			wp_localize_script( 't42-smoothscroll-admin', 't42SmoothScroll', [
				'messages' => $messages
			] );
		}

		/** Clear old messages. */
		MessageHelper::get_instance()->clear_messages();

	}

	/**
	 * Run when the plugin is activated.
	 *
	 * @static
	 * @since 2.0.0
	 **/
	public static function on_activation() {

		/** Security checks. */
		if ( ! current_user_can( 'activate_plugins' ) ) { return; }

		/** We need to know plugin to activate it. */
		if ( ! isset( $_REQUEST['plugin'] ) ) { return; }

		/** Get plugin. */
		$plugin = filter_var( $_REQUEST['plugin'], FILTER_SANITIZE_STRING );

		/** Checks that a user was referred from admin page with the correct security nonce. */
		check_admin_referer( "activate-plugin_{$plugin}" );

		/** Do critical initial checks. */
		if ( ! CheckCompatibility::get_instance()->do_initial_checks( false ) ) { return; }

		/** Send install Action to our host. */
		self::send_install_action();

	}

	/**
	 * Send install Action to our host.
	 *
	 * @static
	 * @since 2.0.1
	 **/
	private static function send_install_action() {

		/** Plugin version. */
		$ver = self::get_instance()->get_version();

		/** Have we already sent 'install' for this version? */
		$opt_name = 't42_smoothscroll_send_action_install';
		$ver_installed = get_option( $opt_name );

		/** Send install Action to our host. */
		if ( ! $ver_installed || $ver !== $ver_installed ) {

			/** Send install Action to our host. */
			Helper::get_instance()->send_action( 'install', 't42-smoothscroll', $ver );
			update_option( $opt_name, $ver );

		}

	}

	/**
	 * Main SmoothScroll Instance.
	 *
	 * Insures that only one instance of SmoothScroll exists in memory at any one time.
	 *
	 * @static
	 * @return SmoothScroll
	 * @since 1.1.0
	 **/
	public static function get_instance() {

		/** @noinspection SelfClassReferencingInspection */
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof SmoothScroll ) ) {

			self::$instance = new SmoothScroll;

		}

		return self::$instance;

	}

} // End Class SmoothScroll.

/** Run when the plugin is activated. */
register_activation_hook( __FILE__, [ SmoothScroll::class, 'on_activation' ] );

/** Run Speaker class once after activated plugins have loaded. */
add_action( 'plugins_loaded', [ SmoothScroll::get_instance(), 'setup' ] );
