<?php
/**
 * Smooth Scroll for WordPress. Exclusively on Envato Market: http://codecanyon.net/user/42theme/portfolio?ref=42Theme
 * @encoding     UTF-8
 * @version      2.0.1
 * @copyright    Copyright (C) 2019 42Theme ( https://42theme.com ). All rights reserved.
 * @license      Envato License https://codecanyon.net/licenses/standard?ref=42theme
 * @author       Alexander Khmelnitskiy ( ak@42theme.com )
 * @support      support@42theme.com
 **/

namespace T42;

/** Include plugin autoloader for additional classes. */
require __DIR__ . '/src/autoload.php';

use T42\SmoothScroll\Helper;
use T42\SmoothScroll\EnvatoItem;
use T42\SmoothScroll\Settings;

/** Exit if uninstall.php is not called by WordPress. */
if ( ! defined( 'WP_UNINSTALL_PLUGIN' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Class used to implement Uninstall of SmoothScroll plugin.
 *
 * @since 2.0.0
 * @author Alexandr Khmelnytsky (info@alexander.khmelnitskiy.ua)
 **/
final class Uninstall {

	/**
	 * The one true Uninstall.
	 *
	 * @var Uninstall
	 * @since 2.0.0
	 **/
	private static $instance;

	/**
	 * Sets up a new Uninstall instance.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	private function __construct() {

		/** Get Uninstall mode. */
		$uninstall_mode = $this->get_uninstall_mode();

		/** Send uninstall Action to our host. */
		Helper::get_instance()->send_action( 'uninstall', 't42-smoothscroll', '2.0.1' );

		/** Remove Plugin and Settings. */
		if ( 'plugin+settings' === $uninstall_mode ) {

			/** Remove Plugin Settings. */
			$this->remove_settings();
		}
	}

	/**
	 * Return uninstall mode.
	 * plugin - Will remove the plugin only. Settings and Audio files will be saved. Used when updating the plugin.
	 * plugin+settings - Will remove the plugin and settings. Audio files will be saved. As a result, all settings will be set to default values. Like after the first installation.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function get_uninstall_mode() {

		/** Get plugin settings. */
		$options = Settings::get_instance()->options;

		return $options['delete_plugin'];

	}

	/**
	 * Delete Plugin Options.
	 *
	 * @since 2.0.0
	 * @access public
	 **/
	public function remove_settings() {

		$settings = [
			't42_smoothscroll_envato_id',
			't42_smoothscroll_settings',
			't42_smoothscroll_messages',
			't42_smoothscroll_send_action_install',
			'envato_purchase_code_' . EnvatoItem::get_instance()->get_id() // Plugin item ID.
		];

		foreach ( $settings as $key ) {

			if ( is_multisite() ) { // For Multisite.
				if ( get_site_option( $key ) ) {
					delete_site_option( $key );
				}
			} else {
				if ( get_option( $key ) ) {
					delete_option( $key );
				}
			}
		}
	}

	/**
	 * Main Uninstall Instance.
	 *
	 * Insures that only one instance of Uninstall exists in memory at any one time.
	 *
	 * @static
	 * @return Uninstall
	 * @since 2.0.0
	 * @access public
	 **/
	public static function get_instance() {
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Uninstall ) ) {
			self::$instance = new Uninstall;
		}

		return self::$instance;
	}

}

/** Runs on Uninstall of SmoothScroll plugin. */
Uninstall::get_instance();